@@ -1,6 +1,7 @@
 #!/usr/bin/env python3
 # micropipenv
 # Copyright(C) 2020 Fridolin Pokorny
+# Copyright(C) 2021 Lumir Balhar
 #
 # This program is free software: you can redistribute it and / or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -27,7 +28,7 @@ dependencies using raw pip. The virtual environment is not created, but one can
 issue `python3 -m venv venv/ && . venv/bin/activate` to create one.
 """
 
-__version__ = "1.0.4"
+__version__ = "1.1.0"
 __author__ = "Fridolin Pokorny <fridex.devel@gmail.com>"
 __title__ = "micropipenv"
 
@@ -47,7 +48,7 @@ from pathlib import Path
 from urllib.parse import urlparse
 
 _LOGGER = logging.getLogger(__title__)
-_SUPPORTED_PIP_STR = ">=9,<=21.0.1"  # Respects requirement in setup.py and latest pip to release date.
+_SUPPORTED_PIP_STR = ">=9,<=21.1.2"  # Respects requirement in setup.py and latest pip to release date.
 
 try:
     from pip import __version__ as pip_version
@@ -58,22 +59,22 @@ try:
     try:
         from pip._internal.req import parse_requirements
     except ImportError:  # for pip<10
-        from pip.req import parse_requirements
+        from pip.req import parse_requirements  # type: ignore
 
     try:
         try:
             from pip._internal.network.session import PipSession
         except ImportError:
-            from pip._internal.download import PipSession
+            from pip._internal.download import PipSession  # type: ignore
     except ImportError:
-        from pip.download import PipSession
+        from pip.download import PipSession  # type: ignore
     try:
         from pip._internal.index.package_finder import PackageFinder
     except ImportError:
         try:
-            from pip._internal.index import PackageFinder
+            from pip._internal.index import PackageFinder  # type: ignore
         except ImportError:
-            from pip.index import PackageFinder
+            from pip.index import PackageFinder  # type: ignore
 except Exception:
     _LOGGER.error(f"Check you pip version, supported pip versions: {_SUPPORTED_PIP_STR}")
     raise
@@ -108,6 +109,8 @@ _FILE_METHOD_MAP = OrderedDict(
         ("requirements.txt", "requirements"),
     ]
 )
+__re_nested_vars = re.compile(r"\$\{(?P<name>[^\}:]*)(?::-(?P<default>[^\}]*))?\}")
+__re_sub_vars = re.compile(r"\$\{[^}]*\}")
 
 
 class MicropipenvException(Exception):
@@ -367,7 +370,7 @@ def install_pipenv(
 def _instantiate_package_finder(pip_session):  # type: (PipSession) -> PackageFinder
     """Instantiate package finder, in a pip>=10 and pip<10 compatible way."""
     try:
-        return PackageFinder(find_links=[], session=pip_session, index_urls=_DEFAULT_INDEX_URLS)
+        return PackageFinder(find_links=[], session=pip_session, index_urls=_DEFAULT_INDEX_URLS)  # type: ignore
     except TypeError:  # API changed in pip>=10
         from pip._internal.models.search_scope import SearchScope
         from pip._internal.models.selection_prefs import SelectionPreferences
@@ -381,9 +384,9 @@ def _instantiate_package_finder(pip_session):  # type: (PipSession) -> PackageFi
             from pip._internal.index.collector import LinkCollector
         except ModuleNotFoundError:
             try:
-                from pip._internal.collector import LinkCollector
+                from pip._internal.collector import LinkCollector  # type: ignore
             except ModuleNotFoundError:  # pip>=19.2<20
-                return PackageFinder.create(
+                return PackageFinder.create(  # type: ignore
                     session=pip_session, selection_prefs=selection_prefs, search_scope=search_scope
                 )
 
@@ -411,7 +414,7 @@ def _get_requirement_info(requirement):  # type: (ParsedRequirement) -> Dict[str
     is_url = False
     req = None
     if hasattr(requirement, "req"):
-        req = requirement.req
+        req = requirement.req  # type: ignore
     elif hasattr(requirement, "requirement") and not editable:
         if not requirement.requirement.startswith("git+"):
             req = Requirement(requirement.requirement)
@@ -422,7 +425,7 @@ def _get_requirement_info(requirement):  # type: (ParsedRequirement) -> Dict[str
     link = None
     if editable:
         if hasattr(requirement, "link"):
-            link = str(requirement.link)
+            link = str(requirement.link)  # type: ignore
         elif req is not None:
             link = req.url
         elif hasattr(requirement, "requirement"):
@@ -434,7 +437,7 @@ def _get_requirement_info(requirement):  # type: (ParsedRequirement) -> Dict[str
     if editable:
         name = str(link)
     elif hasattr(requirement, "name"):
-        name = requirement.name
+        name = requirement.name  # type: ignore
     elif req is not None:
         name = req.name
     elif hasattr(requirement, "requirement") and is_url:
@@ -447,8 +450,8 @@ def _get_requirement_info(requirement):  # type: (ParsedRequirement) -> Dict[str
     version_specifier_length = None
     if not editable and not is_url:
         if hasattr(requirement, "specifier"):
-            version_specifier = str(requirement.specifier)
-            version_specifier_length = len(requirement.specifier)
+            version_specifier = str(requirement.specifier)  # type: ignore
+            version_specifier_length = len(requirement.specifier)  # type: ignore
         elif req is not None:
             # pip>=20
             version_specifier = str(req.specifier)
@@ -460,9 +463,9 @@ def _get_requirement_info(requirement):  # type: (ParsedRequirement) -> Dict[str
     hash_options = None
     if not editable and not is_url:
         if hasattr(requirement, "options"):
-            hash_options = requirement.options.get("hashes")
+            hash_options = requirement.options.get("hashes")  # type: ignore
         else:
-            hash_options = requirement.hash_options  # More recent pip.
+            hash_options = requirement.hash_options  # type: ignore
 
     hashes = {}
     for hash_type, hashes_present in hash_options.items() if hash_options else []:
@@ -475,7 +478,7 @@ def _get_requirement_info(requirement):  # type: (ParsedRequirement) -> Dict[str
     markers = None
     if not editable and not is_url:
         if hasattr(requirement, "markers"):
-            markers = requirement.markers
+            markers = requirement.markers  # type: ignore
         elif req is not None:
             markers = req.marker
         else:
@@ -485,7 +488,7 @@ def _get_requirement_info(requirement):  # type: (ParsedRequirement) -> Dict[str
     extras = None
     if not editable and not is_url:
         if hasattr(requirement, "extras"):
-            extras = requirement.extras
+            extras = requirement.extras  # type: ignore
         elif req is not None:
             extras = req.extras
         else:
@@ -549,7 +552,7 @@ def _requirements2pipfile_lock(requirements_txt_path=None):  # type: (Optional[s
         # We add all dependencies to default, develop should not be present in requirements.txt file, but rather
         # in dev-requirements.txt or similar.
         if requirement_name in result:
-            raise RequirementsError("Duplicate entry for requirement {}".format(requirement.name))
+            raise RequirementsError("Duplicate entry for requirement {}".format(requirement.name))  # type: ignore
 
         result[requirement_name] = entry
 
@@ -948,20 +951,21 @@ def _get_index_entry_str(sections, package_info=None):  # type: (Dict[str, Any],
 
     result = ""
     for idx, source in enumerate(sections.get("sources", [])):
+        url = _resolve_nested_variables(source["url"])
         if index_name is None:
             if idx == 0:
-                result += "--index-url {}\n".format(source["url"])
+                result += "--index-url {}\n".format(url)
             else:
-                result += "--extra-index-url {}\n".format(source["url"])
+                result += "--extra-index-url {}\n".format(url)
 
             if not source["verify_ssl"]:
-                result += "--trusted-host {}\n".format(urlparse(source["url"]).netloc)
+                result += "--trusted-host {}\n".format(urlparse(url).netloc)
         else:
             if index_name == source["name"]:
-                result += "--index-url {}\n".format(source["url"])
+                result += "--index-url {}\n".format(url)
 
                 if not source["verify_ssl"]:
-                    result += "--trusted-host {}\n".format(urlparse(source["url"]).netloc)
+                    result += "--trusted-host {}\n".format(urlparse(url).netloc)
 
                 break
 
@@ -973,6 +977,19 @@ def _get_index_entry_str(sections, package_info=None):  # type: (Dict[str, Any],
     return result
 
 
+def _resolve_nested_variables(url):
+    # type: (str) -> str
+    while True:
+        variable = __re_nested_vars.search(url)
+        if not variable:
+            break
+        value = os.getenv(variable["name"])
+        if not value:
+            value = variable["default"] if variable["default"] else ""
+        url = __re_sub_vars.sub(value, url, count=1)
+    return url
+
+
 def _iter_index_entry_str(
     sections, package_info
 ):  # type: (Dict[str, Any], Optional[Dict[str, Any]]) -> Generator[str, None, None]
@@ -986,6 +1003,7 @@ def _iter_index_entry_str(
         return None
 
     for source in sections["sources"]:
+        source["url"] = _resolve_nested_variables(source["url"])
         result = "--index-url {}\n".format(source["url"])
         if not source["verify_ssl"]:
             result += "--trusted-host {}\n".format(urlparse(source["url"]).netloc)
